// Copyright 2023 The Chromium Authors
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef IOS_CHROME_BROWSER_UI_NTP_NEW_TAB_PAGE_HEADER_VIEW_H_
#define IOS_CHROME_BROWSER_UI_NTP_NEW_TAB_PAGE_HEADER_VIEW_H_

#import <UIKit/UIKit.h>

@class GradientView;

// Header view for the NTP. The header view contains all views that are
// displayed above the list of most visited sites, which includes the
// primary toolbar, doodle, and fake omnibox.
@interface NewTabPageHeaderView : UIView

// Returns the toolbar view.
@property(nonatomic, readonly) UIView* toolBarView;

// The Identity Disc showing the current user's avatar on NTP.
@property(nonatomic, strong) UIView* identityDiscView;

// Voice search button.
@property(nonatomic, strong, readonly) UIButton* voiceSearchButton;

// The button that opens Lens. May be nil if Lens is not enabled.
@property(nonatomic, strong, readonly) UIButton* lensButton;

// Fake cancel button, used for animations. Hidden by default.
@property(nonatomic, strong) UIView* cancelButton;
// Fake omnibox, used for animations. Hidden by default.
@property(nonatomic, strong) UIView* omnibox;

@property(nonatomic, strong)
    NSLayoutConstraint* fakeLocationBarLeadingConstraint;
@property(nonatomic, strong)
    NSLayoutConstraint* fakeLocationBarTrailingConstraint;
@property(nonatomic, strong) GradientView* fakeLocationBar;
@property(nonatomic, strong) UILabel* searchHintLabel;

// `YES` if Google is the default search engine.
@property(nonatomic, assign) BOOL isGoogleDefaultSearchEngine;

// Adds the separator to the searchField. Must be called after the searchField
// is added as a subview.
- (void)addSeparatorToSearchField:(UIView*)searchField;

// Adds the `toolbarView` to the view implementing this protocol.
// Can only be added once.
- (void)addToolbarView:(UIView*)toolbarView;

// Returns the progress of the search field position along
// `ntp_header::kAnimationDistance` as the offset changes.
- (CGFloat)searchFieldProgressForOffset:(CGFloat)offset;

// Changes the constraints of searchField based on its initialFrame and the
// scroll view's y `offset`. Also adjust the alpha values for `_searchBoxBorder`
// and `_shadow` and the constant values for the `constraints`. `screenWidth` is
// the width of the screen, including the space outside the safe area. The
// `safeAreaInsets` is relative to the view used to calculate the `width`.
- (void)updateSearchFieldWidth:(NSLayoutConstraint*)widthConstraint
                        height:(NSLayoutConstraint*)heightConstraint
                     topMargin:(NSLayoutConstraint*)topMarginConstraint
                     forOffset:(CGFloat)offset
                   screenWidth:(CGFloat)screenWidth
                safeAreaInsets:(UIEdgeInsets)safeAreaInsets;

// Adds views necessary to customize the NTP search box.
- (void)addViewsToSearchField:(UIView*)searchField;

// Highlights the fake omnibox.
- (void)setFakeboxHighlighted:(BOOL)highlighted;

@end

#endif  // IOS_CHROME_BROWSER_UI_NTP_NEW_TAB_PAGE_HEADER_VIEW_H_
